#include "vvl_defs.h"
#include "internet.h"
#include "mcam.h"
#include "cpia.h"
#include "effects.h"

extern int debug_flag;
extern char *frame_dir;


int p_effect(BYTE __huge *buffer, int in_size)
{
int a, c, i, j;
int low_byte, high_byte;
int line_length;
BYTE __huge *ptr;
int line_cnt;
int luminance_matrix[288][352];
int u_matrix[144][176];
int v_matrix[144][176];
int y_val, u_val, v_val;
int y_width, u_width, v_width;
int y_replace, u_replace, v_replace;
FILE *fptr;
char temp[1024];

/* cannot process compressed buffer */
if(buffer[COMP_ENABLE] == 1)
	{
	fprintf(stderr,\
	"p_effect(): No special effects when compression is on\n");
	return 1;
	}

sprintf(temp, "%s/effects.dat", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr, "process_yuyv(): cannot open file for read\n");
	return 0;
	}

fscanf(fptr,\
"%d %d %d  %d %d %d  %d %d %d",\
&y_val, &u_val, &v_val,\
&y_width, &u_width, &v_width,\
&y_replace, &u_replace, &v_replace);

fclose(fptr);

/*
scip the header, point to first line.
format is line length low, line length high, YUYV, EOL (0xfd), 
frame ends with 0xff 0xff 0xff 0xff
*/
ptr = buffer + 64;
line_cnt = 0;
while(1)
	{
	/* low byte line length */
	low_byte = *ptr;
	ptr++;

	/* high byte line length */
	high_byte = *ptr;
	ptr++;

	/* get length of this line */
	line_length = low_byte + (256 * high_byte);

	if(debug_flag)
		{
		fprintf(stderr, "p_compress(): line=%d line_length=%d\n",\
		line_cnt, line_length);
		}

	/* test for last line (255 in both low and high position) line length */
	if(line_length == 65535) break; 

	/* process the line */
	i = 0;
	for(i = 0; i < line_length / 4; i++)
		{
		/* first y */
		/* process y */
		process_yuyv(ptr, ptr + 1, ptr + 2, ptr + 3,\
		y_val, u_val, v_val,\
		y_width, u_width, v_width,\
		y_replace, u_replace, v_replace);

		ptr++;

		/* u (blue vector) 255 is blue, 127 is no color, 0 is yellowish */
		/* process u */
		ptr++;

		/* second y */
		/* process y */
		ptr++;

		/* v (red vector) 255 is red, 127 = no color, 0 is greenish */
		/* process v */
		ptr++;
		}/* end for each line */
	ptr++;

	line_cnt++;
	}/* end for all lines */

if(debug_flag)
	{
	fprintf(stderr, "p_effect(): last line detected\n");
	}

return 1;
}/* end function p_effect */


int process_yuyv(\
BYTE __huge *y1, BYTE __huge *u, BYTE __huge *y2, BYTE __huge *v,\
int y_val, int u_val, int v_val,\
int y_width, int u_width, int v_width,\
int y_replace, int u_replace, int v_replace)
{
/* u (blue vector) 255 is blue, 127 is no color, 0 is yellowish */
/* v (red vector) 255 is red, 127 = no color, 0 is greenish */
int a, b, c, j, i;
int y1_flag, y2_flag, u_flag, v_flag;

/* luminance comparator */
y1_flag = 0;
y2_flag = 0;
if( (*y1 > (y_val - (y_width / 2) ) ) &&\
(*y1 < (y_val + (y_width / 2) ) ) )
	{
	y1_flag = 1;
	}
if( (*y2 > (y_val - (y_width / 2) ) ) &&\
(*y2 < (y_val + (y_width / 2) ) ) )
	{
	y2_flag = 1;
	}

if(y1_flag && y2_flag)
	{
	*y1 = y_replace;
	*y2 = y_replace;
	}

/* chrominance comparator */
u_flag = 0;
if( (*u > (u_val - (u_width / 2) ) ) &&\
(*u < (u_val + (u_width / 2) ) ) )
	{
	u_flag = 1;
	}

v_flag = 0;
if( (*v > (v_val - (v_width / 2) ) ) &&\
(*v < (v_val + (v_width / 2) ) ) )
	{
	v_flag = 1;
	}

if(u_flag && v_flag)
	{
	*u = u_replace;
	*v = v_replace;
	}

return 1;
}/* end function process_yuyv */


int detect_change(\
BYTE __huge *buffer, int in_size, int *change_flag,\
int y_difference, int u_difference, int v_difference, int treshhold)
{
int a, c, i, j;
int low_byte, high_byte;
int line_length;
BYTE __huge *ptr;
int line_cnt;
int y_val, u_val, v_val;
int y_width, u_width, v_width;
int y_replace, u_replace, v_replace;
FILE *fptr;
char temp[1024];
int y_idx, u_idx, v_idx;
int change_level;
int y_matrix[288][356];
int u_matrix[288][178];
int v_matrix[288][178];
static int old_y_matrix[288][356];
static int old_u_matrix[288][178];
static int old_v_matrix[288][178];
int diff;

if(debug_flag)
	{
	fprintf(stderr,\
	"detect_change(): buffer=%lu in_size=%d\n\
	y_difference=%d u_difference=%d v_difference=%d treshhold=%d\n",\
	(long)buffer, in_size,\
	y_difference, u_difference, v_difference, treshhold);
	}

/* set for no change in case error return */
*change_flag = 0;

/* cannot process compressed buffer */
if(buffer[COMP_ENABLE] == 1)
	{
	fprintf(stderr,\
	"detect_change(): Only works with compression off\n");
	return 1;
	}

/* fill the new matrices */

/*
scip the header, point to first line.
format is line length low, line length high, YUYV, EOL (0xfd), 
frame ends with 0xff 0xff 0xff 0xff
*/
ptr = buffer + 64;
line_cnt = 0;
while(1)
	{
	/* low byte line length */
	low_byte = *ptr;
	ptr++;

	/* high byte line length */
	high_byte = *ptr;
	ptr++;

	/* get length of this line */
	line_length = low_byte + (256 * high_byte);

	if(debug_flag)
		{
		fprintf(stderr, "detect_change(): line=%d line_length=%d\n",\
		line_cnt, line_length);
		}

	/* test for last line (255 in both low and high position) line length */
	if(line_length == 65535) break; 
	/* index in array */
	y_idx = 0;
	u_idx = 0;
	v_idx = 0;
	/* process the line */
	i = 0;
	for(i = 0; i < line_length / 4; i++)
		{
		/* first y */
		/* to luminance matrix */
		y_matrix[line_cnt][y_idx] = *ptr;
		ptr++;
		y_idx++;
		
		/* u (blue vector) 255 is blue, 127 is no color, 0 is yellowish */
		/* to u matrix */
		u_matrix[line_cnt][u_idx] = *ptr;
		ptr++;
		u_idx++;
		
		/* second y */
		/* to luminance matrix */
		y_matrix[line_cnt][y_idx] = *ptr;
		ptr++;
		y_idx++;

		/* v (red vector) 255 is red, 127 = no color, 0 is greenish */
		/* to v matrix */
		v_matrix[line_cnt][v_idx] = *ptr;
		ptr++;
		v_idx++;
		}/* end for each line */
	ptr++;

	line_cnt++;
	}/* end for all lines */

/* set for no change */
change_level = 0;

/* compare new to old y matrices */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < y_idx; j++)
		{
		diff = y_matrix[i][j] - old_y_matrix[i][j];
		if( abs(diff) > y_difference)
			{
			change_level++;
			}
		}
	}

/* compare new to old u matrices */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < u_idx; j++)
		{
		diff = u_matrix[i][j] - old_u_matrix[i][j];
		if( abs(diff) > u_difference)
			{
			change_level++;
			}
		}
	}

/* compare new to old v matrices */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < v_idx; j++)
		{
/*		diff = v_matrix[i][j] != old_v_matrix[i][j];*/
		diff = v_matrix[i][j] - old_v_matrix[i][j];
		if( abs(diff) > v_difference)
			{
			change_level++;
			}
		}
	}

/* update old y matrix */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < y_idx; j++)
		{
		old_y_matrix[i][j] = y_matrix[i][j];
		}
	}
	
/* update old u matrix */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < u_idx; j++)
		{
		old_u_matrix[i][j] = u_matrix[i][j];
		}
	}

/* update old v matrix */
for(i = 0; i < line_cnt; i++)
	{
	for(j = 0; j < v_idx; j++)
		{
		old_v_matrix[i][j] = v_matrix[i][j];
		}
	}

if(debug_flag)
	{
	fprintf(stderr, "detect_change(): change_level=%d\n", change_level);	
	}

/* set flag if treshhold exceeded */
if(change_level > treshhold) *change_flag = 1;

return 1;
}/* end function detect_change */


 
